C GaussNewton.for
C
C This file contains the source code and module
C defintions that implement the damped Gauss-Newton algorithm
C explained in Chapter 2.2.2 of Heer and Maussner

C The module GNPars holds the parameters used by the
C algorithm. You can change any of them by changing
C the parameter asignment.

      Module GNPars

	Integer(4), Parameter :: GN_Maxit=500 ! maximum number of iterations
	Real(8), Parameter :: ParTol=1.0e-10  ! convergence criterium 2
	Real(8), Parameter :: GradTol=1.0e-6  ! convergence criterium 1
	Integer(4) FError

	End Module GNPars

C GaussNewton
C
C Solves a non-linear least squares problem using
C the damped Gauss-Newton algorithm
C
C Usage: Call GaussNewton(m,n,x0,typx,typS,f,df,x1,rc)
C
C Input:   m, Integer(4), number of parameters
C          n, Integer(4), numboer of observations
C         x0, Real(8), m times 1 vector, starting parameters
C       typx, Real(8), m timex 1 vector, the typical elements of x
C       typS, Real(8), the typical size of S
C          f, pointer to the function that returns y-F(.,x),
C             usage must be Call f(m,n,,x, fx), where fx=y-F(.,x) is
C             a Real(8) n times 1 vector
C         df, pointer to the function that returns the
C             gradient of F(.,x) with respect to x, usage
C             must be Call df(m,n,x,xx), where xx is a Real(8)
C             n times m matrix.
C
C Output: x1, Real(8), m times 1 vector, the final parameter estimate
C         rc, Integer(4), error code
C             where: rc=1: it was not possible to solve the linear system
C                    rc=2: step size smaller than step tolerance but
C                          gradient different from zero
C                    rc=3: parameter estimate converged to non-optimal point
C                    rc=4: maximum number of iterations exceeded
C                    rc=5: it was not possible to evaluate the function f

            

	SUBROUTINE GaussNewton(m,n,x0,typx,typS,f,df,x1,rc)

      use GNPars
      use DFLIB
      use IMSLF90

	Integer(4) m, n, rc
	Real(8) x0(m), typx(m), x1(m), typS

	INTEGER(4) I, step, fh	
	INTEGER(2) Inkey
      Real(8) x2(1:m), dx(1:m), xy(1:n), xx(1:n,1:m), bvec(1:m), 
     +        amat(1:m,1:m), sx, gradS(1:m), 
     +        converge1, converge2, lambda1, lambda2
	      
      External f, df

      TYPE (rccoord) curpos	

C Initialize
	      
	fh=GetActiveQQ() ! Handle of active window to write messages
      rc=0             ! set return code to 0
       
      
      x1=x0
	dx=x0
	lambda=1.0     

      
C Change Error Behavior of DLSADS
C      Call Erset(0,0,0)      
      Call UMACH(-3,fh)
      FError=0
      Call  f(m,n,x1,xy) ! vector of errors
	if (FError .ne. 0) then
	  rc=5
	  return
	endif
	Call df(m,n,x1,xx) ! matrix xbar (gradient of f at x1)
	sx=Dot_Product(xy,xy) ! sum of squared errors
	sx=dble(sx/n)
	
	gradS=MATMUL(Transpose(xx),xy)
	gradS=-2.0*dble(gradS/n) ! gradient of S(x), the sum of squared errors
      	
C Gauss-Newton Iterationens
	do Step=1,GN_Maxit,1
C         write(15,"('GNIteration=',I4)") Step
C	   Call DLSVRR(n,m,xx,n,ipath,tol,irank,smat,umat,n,vmat,m)
         Bvec=MATMUL(TRANSPOSE(xx),xy)
	   Amat=MATMUL(TRANSPOSE(xx),xx)	   
         CALL DLSADS(m, Amat, m, Bvec, dx) !compute new vector
C         Write(15,"('Error Code from DLSADS=',I1)") IERCD()
         Select Case (IERCD())
	          Case(1)
	           Call SetTextposition(20,2,curpos)
	           WRITE(fh,"('Warning: ill conditioned matrix')")
	           WRITE(15,"('Warning: ill conditioned matrix')")
	          Case(2)
                  rc=1
	            exit
	   End Select
	   lambda1=1.0
	   FError=0
	   Call  f(m,n,x1+lambda1*dx,xy) ! vector of errors
	   do while (FError .ne. 0)
	      lambda1=0.5*lambda1
	      FError=0
            Call  f(m,n,x1+lambda1*dx,xy)
	   end do
	   
	   dx=lambda1*dx
	   CALL GNStep(m,n,x1,dx,f,gradS,typx,lambda2,rc) ! compute stepsize
C	   Write(15,"('Stepsize= ',F20.15,' rc=',I1)") lambda2, rc
         dx=lambda2*dx	   
         x2=x1+dx	   
         Call  f(m,n,x2,xy)
	   sx=Dot_Product(xy,xy)
	   sx=dble(sx/n)
	   Call df(m,n,x2,xx)
         gradS=MATMUL(Transpose(xx),xy)
	   gradS=-2.0*dble(gradS/n)
         converge1=ConTest2(m,gradS,x2,sx,typx,typS)
	   converge2=ConTest1(m,x1,x2,typx)
C         Write(15,"('c1= ',F15.10,' c2= ',F15.10)") Converge1, converge2
	   CALL SETTEXTPOSITION(18,2,curpos)    ! write message
	   WRITE(fh,"('GN-Iteration No =',I6)") Step
	   CALL SetTextPosition(19,2,curpos)
	   WRITE(fh,"('s1=',E10.4,' s2=',E10.4,' c1=',F10.6,2X,' c2=',
     +              F10.7)") lambda1, lambda2, converge1, converge2
	   if (converge1 .lt. GradTol) then 	   
	      rc=0
	     x1=x2
		  exit	      
	   elseif (rc .eq. 1) then
	      rc=2
	      exit
	   elseif(converge2 .lt. ParTol) then
	      rc=3
	      exit
	   endif
	  
	   x1=x2            

      End Do

      If (Step .gt. GN_Maxit) rc=4
      
C Reset Error Behavior of DLSADS
      Call Erset(0,2,2)
C	Write(15,"('Exit code from GN=',I1)") rc
      Return
	End Subroutine

C GNStep: Find the step size for the Gauss-Newton Iterations
C           
C Usage: Call GNStep(m,n,x,dx,f,df,s,rc)
C        Call GNStep(m,n,x,dx,f,gradS,typx,s,rc)
C
C Input: m, Integer(4), number of parameters
C        n, Integer(4), number of observations
C        x, Real(8), m times 1 vector of paramters
C       dx, Real(8), m times 1 vector, change of parameters
C        f, pointer to the function that returns y-F(.,x) in
C           a n times 1 vector
C    gradS, Real(8), m times 1 vector, the gradient of S(x)
C     typx, Real(8), m times 1 vector, the typical size of x(i)
C
C Ouput: s, Real(8), the step size
C        rc, Integer(4) return code: 0=normal termination,
C                                    1=s below step tolerance 
C
C Remark: if s is found to be smaller than ParTol, s=-1 is
C         returned
         
      Subroutine GNStep(m,n,x,dx,f,gradS,typx,s,rc)

      use GNPars
      use IMSLF90 	

	Integer(4) m, n, rc	
	Real(8) x(m), dx(m), typx(m), gradS(m), s

	Real(8) x1(m), s1, s2, smult, smin, smax, amat(2,2), bvec(1:2,1),
     +        ab(1:2,1), xy(1:n), xx(1:n,1:m), fx0, fx1, fx2, 
     +        dfs, ConTest1, StepTol

	External f


C Initialize      
	smult=1.0e-4	
	smin=0.10
	smax=0.5
      
      s1=1.0
	rc=0
      StepTol=ConTest1(m,x,x+dx,typx)
C	Write(15,"('Maximum element in |dx|/|x|:',F15.10)") StepTol
	StepTol=ParTol/StepTol      
C      write(15,"('StepTol=',F15.10)") StepTol
	StepTol=ParTol

	Call F(m,n,x,xy)
	fx0=Dot_Product(xy,xy)
	fx0=dble(fx0/n)
	dfs=dot_product(GradS,dx) 
	x1=x+dx
	Call f(m,n,x1,xy)
      fx1=Dot_Product(xy,xy)
	fx1=dble(fx1/n)
      
	if (fx1 .gt. (fx0+smult*dfs)) then
	   s=-dfs/(2*(fx1-fx0-dfs))
	   if (s .lt. smin) s=smin
	   if (s .gt. smax) s=smax
	   x1=x+s*dx
	   Call F(m,n,x1,xy)
	   fx2=Dot_Product(xy,xy)
	   fx2=dble(fx2/n)
	else
	   s=1.0	
	   return
	endif
      s2=s
	do while (fx2 .gt. (fx0+smult*s2*dfs)) 
	   Amat(1,1)=1/(s2**2)
	   Amat(1,2)=-1/(s1**2)
	   Amat(2,1)=-s1/(s2**2)
	   Amat(2,2)=s2/(s1**2)
	   Bvec(1,1)=fx2-s2*dfs-fx0
	   Bvec(2,1)=fx1-s1*dfs-fx0
         ab=MatMul(Amat,Bvec)/(s2-s1)
	   if (ab(1,1) .eq. 0.0) then
            s=-dfs/(2*ab(2,1))
	   else
	      s=(-ab(2,1)+dsqrt((ab(2,1)**2)-3*ab(1,1)*dfs))/(3*ab(1,1))
	   endif
	   if (s .lt. s2*smin) s=s2*smin
	   if (s .gt. s2*smax) s=s2*smax
         if (s .lt. StepTol) then
	      rc=1
	      return
	   endif
	   s1=s2
	   s2=s
         fx1=fx2
	   x1=x+s2*dx
	   Call F(m,n,x1,xy)
	   fx2=Dot_Product(xy,xy)
	   fx2=dble(fx2/n)
	
      end do

	End Subroutine

C ConTest1: implements the parameter convergence criterium
C           from Dennis and Schnabel (1983), p. 160
C
C usage: converge=ConTest1(nx,x0,x1,typx)
C
C Input: nx, Integer(4), the number of elements in x0, x1, and typx
C        x0, Real(8) vector with nx elements, the first value of x
C        x1, Real(8) vector with nx elements, the second value of x
C       typ, Real(8) vector with nx elements, the typical values of x(i)
C
C Output: Real(8), the minimal value of the convergence criterium

      Real(8) Function ConTest1(nx,x0,x1,typx) 

	Integer(4) nx
	Real(8) x0(nx), x1(nx), typx(nx)

	Integer(4) i
	Real(8) converge, temp

      converge=0.0

	Do i=1,nx
	   temp=dabs(x1(i)-x0(i))/dmax1(dabs(x1(i)),dabs(typx(i)))
	   if (temp .gt. converge) converge=temp
      end do

	ConTest1=converge

	Return

	End Function

C ConTest2: implements the gradient criterium
C           from Dennis and Schnabel (1983), p. 160
C
C usage: converge=ConTest2(nx,gradf,x,fx,typx,typf)
C
C Input:   nx, Integer(4), the number of elements in x0, x1, and typx
C       gradf, Real(8) vector with nx elements, the gradient of
C              f at x
C           x,  Real(8) vector with nx elements, the first value of x
C          fx, the value of f at x
C        typx, Real(8) vector with nx elements, the typical values of x(i)
C        typf, Real(8) the typical value of fx
C
C Output: Real(8), the minimal value of the convergence criterium

      Real(8) Function ConTest2(nx,gradf,x,fx,typx,typf)
                        
	Integer(4) nx
	Real(8) gradf(nx), x(nx), fx, typx(nx), typf

	Integer(4) i
	Real(8) converge, temp

      converge=0.0
	Do i=1,nx
	   temp=gradf(i)*dmax1(dabs(x(i)),dabs(typx(i)))
	   temp=dabs(temp/dmax1(dabs(fx),dabs(typf)))
         if (temp .gt. converge) converge=temp
      End Do

	ConTest2=converge

	Return

	End Function
