C Ramsey4c:
C
C Purpose: Simulate the prototype real business cycle
C          model. The solution procedure is 
C          parameterized expectations.
C
C          The parameters of the expectations function
C          are found by solving the system of equations
C          whose zero is the fixed point of the mapping
C          that defines the approximate solution.
C 
C          Initial values of these parameters are found
C          by a genetic algorithm. This solution
C          is refined using the non-linear equation solver
C          in the subroutine FixvMN.
C
C The parameters of the model are:
C
C a    =growth factor of labor augmenting technical progress
C alpha=elasticity of production with respect to capital,
C       must be in (0,1)
C beta =discount factor
C delta=rate of capital depreciation,
C       must be in (0,1)
C eta  =elasticity of marginal utility,
C       must be greater than zero
C nstar=share of time devoted to work
C rho  =autoregressive parameter of productivity shock
C simga=standard deviation of innovations to productivity shock
C nobs =length of time series (should be large, e.g. 10,000)
C   ta =the number of observations discarded from the computation of the fixed point
C
C The parameters that determine the programs behavior are
C
C HPLambda=the smoothing parameter of the Hodrick-Prescott Filter
C          see Appendix A1.5 to Chapter 1 for relevant values
C outfile =character variable, the name of the file to write the
C          program's output to
C final   =logical (final=t: compute final time series statistics
C          and write output to outfile, final=f: save
C          parameter estimate to file, but do not compute
C          time series moments)
C initialize= logical (initialize=t: draw shocks, start
C             parameter search and save shocks to disk, f:
C             load shocks from disk)


C First, we define common parameters and vectors in the module def1
      Module Def1

      Integer(4) nobs, ng, ta, nrx, FErrorType
      Real(8) a, alpha, beta, delta, eta, theta, kstar, nstar	
	Real(8) rho, sigma
      Real(8), Allocatable :: nt(:), ct(:), kt(:), zt(:), eps(:)
      Real(8) MaxExp1, MinExp1, MaxExp2
      End Module Def1

C Here comes the code for the main program unit
      Program Ramsey4c

	
      use Def1
	use MNR
      use DFLIB
      use IMSLF90 
      
      Integer(4) npars, search, t, rc, length, lags, nofs, freq(3)
      Integer(2) row
      Real(8) yk, ck, yc, cov(5,5)
      Real(8) HPLambda, moments(5,3), GetSSE, DNRM2	
	Real(8) Par0(6), Par1(6), Par2(6), Fit, Psi
	Real(8) crit(1:4)      
	Real(8), allocatable :: yt(:), wt(:),
     +                        it(:),  xt(:,:), xh(:,:)
      REAL(4) Asec
      
	Integer(2) nums, maxs	
      CHARACTER(256) RunTime
      CHARACTER(50) MyDate
      CHARACTER(15) Vars(5)
      CHARACTER(20) OutFile	
	Logical ok2, initialize, InitialSettings

      External GetFit1, GetFit2, Psi           

      TYPE (rccoord) curpos            

      DATA moments /15*0.0/
      DATA Vars(1)/'Output'/
      DATA Vars(2)/'Investment'/
      DATA Vars(3)/'Consumption'/
      DATA Vars(4)/'Hours'/
      DATA Vars(5)/'Real Wage'/      
      
	
C Read Parameters from File
      Call GetD('a','Parameters.txt',a)
      Call GetD('alpha','Parameters.txt',alpha)
      Call GetD('eta','Parameters.txt',eta)
      Call GetD('delta','Parameters.txt',delta)
      Call GetD('nstar','Parameters.txt',nstar)
      Call GetD('beta','Parameters.txt',beta)
      Call GetD('rho','Parameters.txt',rho)
      Call GetD('sigma','Parameters.txt',sigma)
      Call GetD('hplambda','Parameters.txt',hplambda)
      Call GetI('nobs','Parameters.txt',nobs)
	Call GetI('ta'  ,'Parameters.txt',ta)
	Call GetI('search','Parameters.txt',search)
      Call GetC('outfile','Parameters.txt',outfile)
	Call GetI('length', 'Parameters.txt',length)
	Call GetI('lags',   'Parameters.txt',lags)
	Call GetI('nofs',   'Parameters.txt',nofs)
	Call GetL('initialize','Parameters.txt',initialize)

C Assign hard wired parameters      
      NPars=6  
	nrx=nobs+ta+1

	MaxExp1=2.0
	MaxExp2=10.0
      MaxExp1=dlog(MaxExp1)
	MaxExp2=dlog(MaxExp2)
	MinExp1=MaxExp1/MaxExp2
	t=MaxExponent(MinExp1)

	MaxExp2=0.95*dble(t*MinExp1)
	MaxExp2=0.5*MaxExp2
	MaxExp1=0.95*dble(t*MaxExp1)
	t=MinExponent(MinExp1)
	MinExp1=2.0
	MinExp1=dlog(MinExp1)
	MinExp1=0.95*dble(t*MinExp1)

C Compute derived parameters and stationary solution
      yk=((a**eta)-beta*(1-delta))/(beta*(1-alpha)) ! output-capital ratio      
      ck=yk+(1-delta-a)                             ! consumption-capital ratio 
      yc=yk/ck                                      ! inverse of average propensity to consume 
      theta=(alpha*yc*(1-nstar))/nstar              ! theta implied by nstar                
      kstar=nstar*(yk**(-1/alpha))                  ! stationary capital stock      

C Change Appearance of the program window
      ok2=INITIALSETTINGS()

C Create Output Windows      
      Call MakeSmallWindow(10,"Program Progress",2) 
	

C Allocate space
      ALLOCATE(eps(1:nrx),zt(1:nrx),kt(1:nrx),ct(1:nrx-1),
     +       yt(1:nobs),it(1:nobs),nt(1:nrx-1), 
     +       wt(1:nobs),xt(1:nobs,1:5),xh(1:nobs-1,1:2))

C Start
      ASEC=Secnds(0.0)      

C Draw shocks from a standard normal distribution 
      If (initialize) then ! compute shocks and save for later use
  	   Call SetTextPosition(1,1,curpos)
	   WRITE(10,"('Initialize Shocks')")	   
	   Call DRNNOR(nrx,eps) ! Draw shocks from a standard normal
         open(20, form='unformatted',file='eps.bin')
	   write(20) eps(1:nrx)
	   close(20)
	   Call SetTextPosition(1,1,curpos)
	   Write(10,"('                   ')")	   
	
	else ! read shocks from file
	   
	   open(20,form='unformatted',file='eps.bin')
	   read(20), eps(1:nrx)
	   close(20)
  	   
	endif

C Compute sequence of productivity shocks
      ZT(1)=exp(Sigma*eps(1))
      Do t=1,nrx-1
	   ZT(t+1)=(ZT(t)**Rho)*exp(Sigma*eps(t))
	End Do

C Start computations

	maxs=10

      open(15,file=trim(OutFile))
	open(11,file='LogFile.txt')
      WRITE(11,"(A50)") MyDate()
	WRITE(15,"(A50)") MyDate()

      if (Search .eq. 0) then
       open(20,file='ParIni.txt') ! Read initial values from file
	 Do t=1,Npars
	    Read(20,*) Par1(t)
	 end do
       Call FixvMN(Npars,Par1,GetFit1,Par2,rc,crit)
	 goto 7
	endif
		            
      Do nums=1,maxs
	
	Select Case(search)
        Case(1) 
	   Call GSearch1(GetFit2,NPars,Par1,Fit)      
        Case(2)
	   Call GSearch2(GetFit2,NPars,Par1,Fit)
	End Select
	Call ClearScreen($GCLEARSCREEN)        
	CALL SETTEXTPOSITION(1,2,curpos)
	WRITE(10,"('Simulation No. ',I3)") nums
	WRITE(11,"('Simulation No. ',I3)") nums	
	CALL SETTEXTPOSITION(2,2,curpos)
	WRITE(10,"('Solution from GSearch with Fit=',1X,F15.8)") Fit
	WRITE(11,"('Solution from GSearch with Fit=',1X,F15.8)") Fit	       
	CALL SETTEXTPOSITION(3,2,curpos)
	WRITE(10,*) 'Found Vector:'
	WRITE(11,*) 'Found Vector:'
	row=4		
	DO t=1,NPars
	   CALL SETTEXTPOSITION(row,2,curpos)
	   WRITE(10,"(F12.8)") Par1(t)
	   WRITE(11,"(F12.8)") Par1(t)	      
	   row=row+1
	END DO

      
C Try to refine the estimate using FixvMN
      Call FixvMN(Npars,Par1,GetFit1,Par2,rc,crit)	            
	
	Call SetTextPosition(row,2,curpos)
	WRITE(10,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
	WRITE(11,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
	if (rc .ne. 0) then
	 row=row+1
	 Call SetTextPosition(row,2,curpos)
	 WRITE(10,"('FErrorType: ',I2)") FErrorType
       WRITE(11,"('FErrorType: ',I2)") FErrorType
	 row=row+1
	else
	  exit
	endif
	   
      End Do
	
7	CALL ElapsedTime(SECNDS(Asec),RunTime)
	Call ClearScreen($GCLEARSCREEN)            
      CALL SETTEXTPOSITION(1,2,curpos)
      WRITE(10,"(A100)") RunTime     
      WRITE(11,"(A100)") RunTime     
	WRITE(15,"(A100)") RunTime
	if (nums .gt. maxs) then	 
	   Call SetTextPosition(2,2,curpos)
	   WRITE(10,"('Not able to find solution after ',I2,
     +           'Iterations')") maxs	   
	   WRITE(11,"('Not able to find solution after ',I2,
     +           'Iterations')") maxs	   		  
	  row=row+1
      
	else
	  CALL SetTextPosition(2,2,curpos)
	  WRITE(10,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
	  WRITE(15,"('Error Message from FixvMN: ',A)") MNR_Message(rc)
        Write(15,"('abs(f(i))=',E15.7)") crit(1)
	  Write(15,"('abs(dx/x)=',E15.7)") crit(2)
	  Write(15,"('GradTest =',E15.7)") crit(3)
	  Write(15,"('0.5*f*f  =',E15.7)") crit(4)      
	  Write(15,"('Final Solution')")
	  row=3
	  Do t=1,Npars
	    Call SetTextPosition(row,2,curpos)
	    WRITE(10,"(F15.10)") Par2(t)
	    WRITE(11,"(F15.10)") Par2(t)
	    WRITE(15,"(F15.10)") Par2(t)
	    row=row+1
	  End Do
	  CALL SetTextPosition(row,2,curpos)
	  WRITE(10,"('Compute sum of squared errors')")
	  Fit=GetSSE(Par2,Psi)
	  row=row+1
        CALL SetTextPosition(row,2,curpos)
	  WRITE(10,"('Run DM-Test')")
	  Call Simulate(nofs,lags,length,freq,Npars,Par2,Psi)
	  WRITE(15,"('Sum of squared errors: ', F15.10)") Fit
	  Write(15,"('Total valid Simulations:',I5)") freq(3)
	  Write(15,"('in the lower tail      :',I5)") freq(1)
	  Write(15,"('in the upper tail      :',I5)") freq(2)
	  Write(15,"('Percentage in the lower tail:',F6.2)")
     +     100*dble(freq(1))/dble(freq(3))
	  Write(15,"('Percentage in the upper tail:',F6.2)")
     +     100*dble(freq(2))/dble(freq(3))	   
	  
      endif
      if (rc .eq. 0) then
	 
       if ((eta .eq. 1.00) .and. (delta .eq. 1.00))
     +   then
	   Par0(1)=DLOG(((a/beta)/(1-(1-alpha)*beta))*nstar**(-alpha))
	   Par0(2)=alpha-1.0
	   Par0(3)=0.0E0
	   Par0(4)=-1.0E0
	   Par0(5)=0.0E0
	   Par0(6)=0.0E0
	   Fit=DNRM2(Npars,abs(Par2-Par0),1)/DNRM2(Npars,abs(Par0),1)	   
	   CALL GETTEXTPOSITION(curpos)
	   CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
	   Write(10,"('Euklidian distance to true parameter vector=',
     +   F20.15)") Fit
	   Write(15,"('Euklidian distance to true parameter vector=',
     +   F20.15)") Fit
	   
       endif
       
	 CALL GETTEXTPOSITION(curpos)
	 CALL SETTEXTPOSITION(curpos.row+1,2,curpos)
	 WRITE(10,"('Compute moments')")	
	 Do t=ta,nrx-2
	    yt(t-ta+1)=zt(t)*(nt(t)**alpha)*(kt(t)**(1-alpha))
	    it(t-ta+1)=yt(t-ta+1)-ct(t)
	    wt(t-ta+1)=alpha*(yt(t-ta+1)/nt(t))
       End Do

C Compute percentage deviations from trend
	   Do t=1,nobs
            yt(t)=dlog(yt(t)) 
            it(t)=dlog(it(t)) 
            ct(t)=dlog(ct(ta+t-1)) 
            nt(t)=dlog(nt(ta+t-1)) 
            wt(t)=dlog(wt(t)) 
	   End Do
C Filtering and computation of correlations and standard deviations      
         CALL HPFilter(Nobs,yt,HPLambda,xt(1:nobs,1))
         CALL HPFilter(Nobs,it,HPLambda,xt(1:nobs,2))
         CALL HPFILTER(Nobs,ct,HPLambda,xt(1:nobs,3))
         CALL HPFilter(Nobs,nt,HPLambda,xt(1:nobs,4))
         CALL HPFilter(Nobs,wt,HPLambda,xt(1:nobs,5))
         CALL Corrx(nobs,5,xt,cov)

         moments(1:5,1)=moments(1:5,1)+100*Diagonals(cov)
         moments(2:5,2)=moments(2:5,2)+cov(1,2:5)
    
C Autocorrelations
         Do t=1,5
           xh(1:nobs-1,1)=xt(1:nobs-1,t)
           xh(1:nobs-1,2)=xt(2:nobs,t)      
           CALL Corrx(nobs-1,2,xh,cov(1:2,1:2))
           moments(t,3)=moments(t,3)+cov(1,2)
         End Do
         
          Moments(1,2)=1          
          
C Write solution to file                   
          Do t=1,5 
           WRITE(15,"(A15,3(F7.2,3X))") Vars(t), moments(t,1:3)
          end do    
          WRITE(15,"(A)") 'Column 1: Variable Name'
          WRITE(15,"(A)") 'Column 2: Standard Deviation'
          WRITE(15,"(A)") 'Column 3: Cross Correlation with Output'
          WRITE(15,"(A)") 'Column 4: First Order Autocorrelation'
          WRITE(15,"(A)") ' '
          WRITE(15,"(A)") 'Parameter values:'
          WRITE(15,"('a=       ',F8.4)") a
          WRITE(15,"('alpha=   ',F8.4)") alpha
          WRITE(15,"('beta=    ',F8.4)") beta 
          WRITE(15,"('nstar=   ',F8.4)") nstar
          WRITE(15,"('delta=   ',F8.4)") delta
          WRITE(15,"('rho=     ',F8.4)") rho
          WRITE(15,"('sigma=   ',F8.4)") sigma
          WRITE(15,"('nobs=    ',I6)")   nobs 
	    WRITE(15,"('ta=      ',I6)") ta
	    WRITE(15,"('length=  ',I6)") length
	    WRITE(15,"('lags=    ',I6)") lags
	    WRITE(15,"('nofs=    ',I6)") nofs
	    WRITE(15,"('search=  ',I6)") search
      endif
46	close(11)
      close(15)
	
	   
      
47    END

C *********************** Subroutines ***********************************
C
C Function Psi(Par,Z,K) returns the estimate of
C the right-hand side of the Euler equation, given
C observations on Z and K and the current values of
C the parameters in Par.
C
      Real(8) Function Psi(Par1,K,Z)

	use Def1
	use MNR
	REAL(8) K, Z, Par1(6)
      
	Psi=Par1(1)+Par1(2)*DLOG(K)+Par1(3)*DLOG(K)*DLOG(K)
     +	             +Par1(4)*DLOG(Z)+Par1(5)*DLOG(Z)*DLOG(Z)
     +                 +Par1(6)*DLOG(K)*DLOG(Z)
	
      If (Psi .gt. MaxExp1) then
	   FError=1
	   FErrorType=1	   
	   return
	else if (Psi .lt. MinExp1) then	 
	   FError=1
	   FErrorType=1
	   return	
	endif      


      Psi=beta*(a**(-eta))*exp(Psi)

	Return
	END FUNCTION

C dPsi: computes the gradient of Psi2 in the case of a second degree polynomial
      
	Subroutine dPsi(Gamma,K,Z,g)

      use Def1
      use MNR

	Real(8) Gamma(6), k, z, g(6), Psi

      External Psi

         g(1)=Psi(Gamma,K,Z)
	   g(2)=g(1)*dlog(K)
	   g(3)=g(1)*dlog(K)*dlog(K)
	   g(4)=g(1)*dlog(Z)
	   g(5)=g(1)*dlog(Z)*dlog(Z)
         g(6)=g(1)*dlog(K)*dlog(Z)

      Return

	End Subroutine



C GetFit1: defines the set of equations whose zero is the solution
C          to the fixed point problem that defines the parameter vector
C
      Subroutine GetFit1(NFit,NPar,Par1,Fit)

      use DFLIB      
      use Def1	
      use MNR

	Integer(4) NFit, NPar
	Real(8) Par1(NPar), Fit(NFit)
      	
      Integer(4) t, j
	Real(8) rhs, p1,  n0, g(NPar), Psi, GetN

      
	KT(1)=KSTAR      		
	n0=nstar
      FError=0      

	DO t=1,nrx-1
	   FErrorType=0
	   rhs=Psi(Par1, kt(t), zt(t))
	   If (FerrorType .eq. 1) then
	       FError=1	       
	       return
	   endif
	   FErrorType=0
	   nt(t)=GetN(n0,rhs,kt(t),zt(t))
	   IF (FErrorType .eq. 2) then
	      FError=1	      
	      return
	   endif
	   n0=nt(t)
	   
	   ct(t)=(1-nt(t))*(alpha/theta)*zt(t)*(nt(t)**(alpha-1))*
     +         (kt(t)**(1-alpha))
	 kt(t+1)=(zt(t)*(nt(t)**alpha)*(kt(t)**(1-alpha))+(1-delta)*kt(t)
     +         -ct(t))/a
       if (kt(t+1) .lt. 0.0) then         	      
	      FError=1
		  FErrorType=3			  	      
	      Return
       endif
	   
	End Do 
      
	fit=0.0
	
	Do t=ta,nrx-2
	   p1=(ct(t+1)**(-eta))*((1-nt(t+1))**(theta*(1-eta)))
	   n0=beta*(a**(-eta))*p1*(1-delta+(1-alpha)*zt(t+1)*
     +       (nt(t+1)**alpha)*(kt(t+1)**(-alpha)))		   
	   n0=n0-Psi(Par1, kt(t), zt(t))
	
	   Call dPsi(Par1,kt(t),zt(t),g)
	   do j=1,NFit
	      fit(j)=fit(j)+n0*g(j)
	   end do
	
	End Do		 
	if (MaxVal(dabs(fit)) .gt. (10.**MaxExp2)) then
	  FError=1     
	  FErrorType=4	  
	endif

	return
      
      END SUBROUTINE


C GetFit2(NPar,Par1,Fit)
C 
C Returns the Euklidian Norm of Fit returned by GetFit1

      Subroutine GetFit2(NPar,Par1,Fit)    

      use Def1
	use MNR
C      use IMSLF90 

	Integer(4) NPar
	Real(8) Par1(NPar), Fit
      
	Real(8) Fit2(Npar)      
	
	FError=0
	Call GetFit1(NPar,Npar,Par1,Fit2)

	if (FError .ne. 0) then
	  Fit=-1.0E0
	  return
	endif
C      Fit=DNRM2(Npar,Fit2,1)
      Fit=MaxVal(dabs(Fit2))
      Return
	END SUBROUTINE

C GetN: this function solves the FOC for N
      Real(8) Function GetN(n0,p,k,z)

	use Def1

	Real(8) n0, p, k, z
      
	Integer(4) MaxIt, I
	Real(8) p0, k0, z0, N1, N2, dN, ParTol, crit, s, FOCN, dFOCN

      External FOCN, dFOCN

	Common /pkz/ p0, k0, z0


	p0=p
	k0=k
	z0=z
      FErrorType=0
	ParTol=1.0E-9
	Crit=1.0E-8
	N1=N0
      MaxIt=5000      

	do while ((abs(FOCN(N1)) .gt. crit) .or. (I .gt. MaxIt))

	    dN=-FOCN(N1)/dFOCN(N1)
	    if (dN .gt. 0.0) then
		  if ((N1 + dN) .gt. 1.0) then
	       s=0.99*(1.0-N1)/dN
	       N2=N1+s*dN
	      else
	       N2=N1+dN
	      endif
	    endif
          if (dN .lt. 0.0) then
		   if ((N1 + dN) .lt. 0.0) then		    
	         s=0.99*(-N1)/dN
	        N2=N1+s*dN
	       else
	        N2=N1+dN
	       endif
          endif
	    if (abs((N2-N1)/N2) .lt. ParTol) then
	       GetN=N2
	       return
	    endif
	    N1=N2
	    I=I+1
      End Do
	If (I .gt. MaxIt) FErrorType=2
	GetN=N1
	return

	End Function      

      

C FOCN: the zero of this first order condition delivers N
C
      Real(8) Function FOCN(n)

      use Def1

	Real(8) n
	Real(8) p0, k0, z0

	Common /pkz/ p0, k0, z0

	FOCN=theta*(p0**(-1.0/eta))*((1.0-n)**(theta*((1.0-eta)/eta)-1.0))
     +   -alpha*z0*(n**(alpha-1.0))*(k0**(1.0-alpha))

	Return
	End Function

C dFOCN: the first derivative of FOCN
C
      Real(8) Function dFOCN(n)

      use Def1

	Real(8) n
	Real(8) p0, k0, z0

	Common /pkz/ p0, k0, z0

	dFOCN=-theta*((theta*(1.0-eta)/eta)-1.0)*
     +	(p0**(-1.0/eta))*((1.0-n)**((theta*(1.0-eta)/eta)-2.0))+
     +   (1.0-alpha)*alpha*z0*(n**(alpha-2.0))*(k0**(1-alpha))

	Return
	End Function


      Real(8) Function GetSSE(Gamma,Psi)

	use Def1	      
	
	Real(8) Gamma(ng)
    
      Integer(4) t
	Real(8) rhs, n0, p1, psi, GetN

	External Psi, GetN
      
	KT(1)=KSTAR      		
	n0=nstar      

	do t=1,nrx-1

		   rhs=Psi(Gamma, kt(t), zt(t))		   
	   nt(t)=GetN(n0,rhs,kt(t),zt(t))	   
	   n0=nt(t)
	   
	   ct(t)=(1-nt(t))*(alpha/theta)*zt(t)*(nt(t)**(alpha-1))*
     +         (kt(t)**(1-alpha))
	 kt(t+1)=(zt(t)*(nt(t)**alpha)*(kt(t)**(1-alpha))+(1-delta)*kt(t)
     +         -ct(t))/a
         
	End Do       
	 
	GetSSE=0.0

	do t=ta,nrx-2
	   p1=(ct(t+1)**(-eta))*((1-nt(t+1))**(theta*(1-eta)))
	   n0=beta*(a**(-eta))*p1*(1-delta+(1-alpha)*zt(t+1)*
     +       (nt(t+1)**alpha)*(kt(t+1)**(-alpha)))		   
	   n0=n0-Psi(Gamma, kt(t), zt(t))
	
	   GetSSE=GetSSE+(n0**2.0)

      end do
      GetSSE=GetSSE/dble(nobs)
	Return

	End Function


      Subroutine Simulate(nofs,lags,length,freq,m,Gamma,Psi)
	                         
      use Def1
	use MNR

	Integer(4) nofs, lags, freq(3), m, length
      Real(8) Gamma(m), Psi

      Integer(4) t, i, t1
	Real(8) rhs, dm, chi(2), n0, GetN

	External Psi, GetN

	freq=0
	t1=length+ta+lags

C Loop over number of simulations
      Do i=1, nofs,1

C Draw shocks
	   Call DRNNOR(t1+1,eps) ! Draw shocks from a standard normal
	   ZT(1)=exp(Sigma*eps(1))
         Do t=1,t1
	      ZT(t+1)=(ZT(t)**Rho)*exp(Sigma*eps(t))
	   End Do


C Generate Time Series

      KT(1)=KSTAR      		
	n0=nstar      

      do t=1,t1

		   rhs=Psi(Gamma, kt(t), zt(t))		   
	   nt(t)=GetN(n0,rhs,kt(t),zt(t))	   
	   n0=nt(t)
	   
	   ct(t)=(1-nt(t))*(alpha/theta)*zt(t)*(nt(t)**(alpha-1))*
     +         (kt(t)**(1-alpha))
	 kt(t+1)=(zt(t)*(nt(t)**alpha)*(kt(t)**(1-alpha))+(1-delta)*kt(t)
     +         -ct(t))/a
	  if (kt(t+1) .lt. 0.0) goto 2
         
	End Do       
      
	freq(3)=freq(3)+1

      chi(1)=0.05
	chi(2)=0.95

      Call DMStat(lags,length,dm,chi)

      if (dm .lt. chi(1)) freq(1)=freq(1)+1
	if (dm .gt. chi(2)) freq(2)=freq(2)+1

2     continue
	
    
      End Do

	Return

	End Subroutine
      
	
C DMStat

      Subroutine DMStat(l,length,dm,chi)
                       
	use Def1
	use IMSLF90

	Integer(4) l, length
	Real(8) DM(1,1), Chi(1:2), p(1:2), f1, f2

	Integer(4) t, t1, j, m
	Real(8) evec(1:length,1), x(1:length,1+3*l), Amat(1:3*l+1,1:3*l+1),
     +        Bvec(1:3*l+1,1), Avec(1:3*l+1,1), Help(1:3*l+1,1), DCHIIN

      m=1+3*l
      t1=length+ta+l-1

      Do t=ta+l,t1
         
	   f1=(ct(t+1)**(-eta))*((1-nt(t+1))**(theta*(1-eta)))
	   f2=beta*(a**(-eta))*f1*(1-delta+(1-alpha)*zt(t+1)*
     +       (nt(t+1)**alpha)*(kt(t+1)**(-alpha)))		   
         f1=(ct(t)**(-eta))*((1-nt(t))**(theta*(1-eta)))
	   evec(t-ta-l+1,1)=f2-f1
	   
         x(t-ta-l+1,1)=1.0
	   Do j=1,l
	      x(t-ta-l+1,1+j)=ct(t-j)
	      x(t-ta-l+1,1+l+j)=zt(t-j)
	      x(t-ta-l+1,1+2*l+j)=nt(t-j)
	   End Do

	End Do

      Avec=MatMul(Transpose(x),evec)
      Amat=0
	Do t=1,length
	   Help(1:m,1)=x(t,1:m)
	   Amat(1:m,1:m)=Amat(1:m,1:m)+(evec(t,1)**2)*
     +   MatMul(Help,Transpose(Help))
	End Do

	Call DLINRG(m,Amat,m,Amat,m)

      Bvec=MatMul(Amat,Avec)
	DM=MatMul(Transpose(Avec),Bvec)
     
	p(1)=chi(1)
	p(2)=chi(2)
      
	chi(1)=DCHIIN(p(1),dble(m))
	chi(2)=DCHIIN(p(2),dble(m))

	Return

	End Subroutine
