C Search2.for: Implements a Genetic Algorithm with the following properties:
C
C The Algorithm looks for a vector of parameters Par(Npar) that
C provides the smallest value returned by a criterium function Crit(Npar,Par,fit).
C Fit is assumed to be greater to zero with lower bound zero. If
C crit returns fit<0 this is considered an invalid solution and
C will be discarded.
C
C The initial population is generated by random draws from
C a normal distribution with mean zero and standard deviation sigma
C (a parameter supplied by the user in the file GSP2.txt).
C
C For selection=1 the routine chooes at random (with replacement)
C four parents. Those two with the smallest values of fit will
C have two offspring. The two fittest members of the family pass
C to the next generation.
C
C For selection=2 the routine uses stochastic universal sampling
C to find npop parents. Each pair of parents will have two offspring.
C If those two are valid solutions they pass to the next generation.
C Their parents are discarded.
C
C Both selection scheme use elitism.
C
C 
C There are 5 crossover operators available:
C BLX-alpha with alpha=0.5
C Linear
C Arithmetic
C Shuffle
C Singe Point
C
C
C The user can either select a specific operator to be used throughout
C by putting crossover=1,2,3,4, or 5 or indicate that
C all operators shall be used with equal probability by putting
C crossover=6 in GSP2.txt.
C 
C Mutations occur with probability
C
C probm=mu1+mu2/t,
C 
C and are non-uniform with parameter mu3.
C
C The user can choose mu1-mu3 in GSP2.txt.

C If no user-supplied parameters are available (the file GSP2.txt is not
C in the same directory as the exetuable) the defaul values specified in
C Module GSPars2 are used.
C

      Module GSPars2

	  INTEGER(4) Npop, Ngen, Selection, crossover
	  REAL(8) Probco, mu1, mu2, mu3, sigma	  

	  DATA Npop/50/       ! size of population
	  DATA Ngen/150/      ! number of generations to compute
	  DATA Probco/0.950/  ! probability of crossover
	  DATA mu1/0.250/mu2/0.330/mu3/2.0/sigma/1.0/
        DATA Selection/2/Crossover/6/

	END MODULE GSPars2

C Search2.for: The source code of the subroutine that
C              implements a genetic search algorithm
C              to find the minimum of a user specified
C              function
C
C CALL GSearch2(Crit,Npar,Par,Fit)
C
C where Crit(Npar,Par,Fit) is the pointer to a  routine
C returning the fitness Fit of the parameter vector Par with Npar
C Elements.
C
C The Parameters that determine the performance of the algorithm
C are initialized in the module GSPars. At start, the program
C looks for a file GSP2.txt. If this file is found in the
C directory where the main executable is located, the
C subroutine reads the parameters from this file. Otherwise,
C the default values from GSPars are used.
C
C 
C Te probability of mutations (ProbMut) is
C calculated from
C
C ProbMut(itnum)=mu1 + (mu2/itnum)
C
C and the change of an element choosen for mutation
C is given by
C 
C add=lambda*(1-var1**( (1-(itnum/Ngen))**mu3 )
C
C where lambda is a random draw from the standard normal distribution
C var1 is a random draw from the uniform distribution over [0,1]
C and add is added (subtracted) to an element if another draw from the uniform
C distribution var1 is > 0.5 (<0.5). 


      Subroutine GSearch2(Crit,Npar,Par,Fit)

      Use DFLIB
      use IMSLF90 
	use GSPars2

	INTEGER(4)  P(4), Sort1(1:4), Sort2(1:3), GMaxMin(1:2,1:2)
	INTEGER(4), Allocatable :: Parents(:)
	INTEGER(4)  I, J, K, G, Npar, itnum, fh
	INTEGER(4) MutNum(3), IDMIN, IDMAX
	REAL(8)   Par(Npar), Fit
      REAL(8), Allocatable ::  Genes0(:,:)
	REAL(8), Allocatable ::  Genes1(:,:)
	REAL(8)  Stat(1:4),  FitSum
	REAL(8)  SelectP(1:4,1:Npar+3)
	REAL(8)  Help(1:3,1:Npar+1)
	REAL(8)  P1(1:Npar+1), P2(1:Npar+1)
	REAL(8)  C1(1:Npar+2), C2(1:Npar+2), C3(1:Npar+2)
	REAL(8)  choose, lambda, mutvars(2), add, pmin, pmax, delta
      REAL(8)  probmut
	Logical  ishere
	EXTERNAL Crit      
		
	TYPE (rccoord) curpos

C Definition of the function that computes the probability of mutations at
C generation itnum
      ProbMut(itnum)=mu1+DBLE(mu2/itnum)

C Read parameters from GSP.txt (if here)
      Inquire(File='GSP2.txt',Exist=ishere)
	if (ishere) then
	   Call GetI('npop','GSP2.txt',npop)
	   Call GetI('ngen','GSP2.txt',ngen)
	   Call GetD('probco','GSP2.txt',Probco)
	   Call GetD('mu1   ','GSP2.txt',mu1)
	   Call GetD('mu2   ','GSP2.txt',mu2)
         Call GetD('mu3   ','GSP2.txt',mu3)
	   Call GetD('sigma ','GSP2.txt',sigma)         
	   Call GetI('selection','GSP2.txt',selection)
	   Call GetI('crossover','GSP2.txt',crossover)
      endif
      
	Allocate(Parents(1:npop))
	Allocate(Genes0(1:Npop,1:Npar+2))
	Allocate(Genes1(1:Npop,1:Npar+2)) 

C Open Output Window
      fh=GetActiveQQ() ! Handle of active window to write messages
      
	Call ClearScreen($GCLEARSCREEN)
	CALL SETTEXTPOSITION(1,1,curpos)
	WRITE(fh,*) 'Algortihm GA2'
	CALL SETTEXTPOSITION(2,1,curpos)
	WRITE(fh,*) 'Initialize first generation:'

C Select random number generator
      CALL RNOPT(4)

C Initialize Gene Pool     
      Do I=1,Npop	   
	   CALL SETTEXTPOSITION(2,33,curpos)
	   WRITE(fh,"('Gene #: ',I4)") I
	   CALL DRNNOR(Npar, Par)	
	   Par=sigma*Par   	   
	   CALL Crit(Npar,Par,Fit)	 
	   j=1  	   
	   Do While (Fit .lt. 0.0)
	      j=j+1
	      CALL DRNNOR(Npar, Par)	   	      
	      CALL Crit(NPar,Par,Fit)
	      CALL SetTextPosition(2,45,curpos)
	      WRITE(fh,"('J=',I4)") j
	   End Do
	   Genes0(I,1:Npar)=Par(1:Npar)	   	   	   	   
	   Genes0(I,NPar+1)=Fit
	End Do   
      CALL MyStat(Npop,Genes0(1:npop,Npar+1),Stat)
	CALL SETTEXTPOSITION(5,2,curpos)
      Write(fh,"('Minimum:            ',E20.4E3)") Stat(1)
	CALL SETTEXTPOSITION(6,2,curpos)
	Write(fh,"('Maximum:            ',E20.4E3)") Stat(2)
      CALL SETTEXTPOSITION(7,2,curpos)
	Write(fh,"('Standard deviation: ',E20.4E3)") Stat(4)

C genetic search:

      Do itnum=1,Ngen,1
	MutNum=0	
		
      CALL SETTEXTPOSITION(9,2,curpos)
	Write(fh,"('Generation #: ',I5)") itnum

      CALL SETTEXTPOSITION(12,2,curpos)
	Write(fh,*) 'Old:  '
	CALL SETTEXTPOSITION(13,2,curpos)
	Write(fh,*) 'Joung:'
	CALL SETTEXTPOSITION(11,25,curpos)
	Write(fh,*) 'Best: '
	CALL SETTEXTPOSITION(11,55,curpos)
	Write(fh,*) 'Worst:'

C Select Parents by stochastic universal sampling
      If (selection .eq. 2) then

	 
C Rescale           
      
	   Do I=1,npop
	     Genes0(I,Npar+2)=1.0/Genes0(I,Npar+1)
	   End Do

	   FitSum=dble(Sum(Genes0(1:Npop,Npar+2))/npop)

	   Do I=1,npop
	     Genes0(I,Npar+2)=Genes0(I,Npar+2)/FitSum
         End Do

C Select  parents
        Call DRNUN(1, lambda) !Implements stochastic universal sampling
  	  FitSum=0.0
	  k=1
	  Do I=1,npop
	      FitSum=FitSum+Genes0(I,Npar+2)
            do while (FitSum .gt. lambda)
              Parents(k)=i
	        k=k+1
	        lambda=lambda+1.0
	      end do
        End Do

      endif

      Do G=1,npop,2
      
	if (selection .eq. 1) then
	  CALL RNUND (4, Npop, P)
        Do I=1,4,1
          SelectP(I,1:Npar+1)=Genes0(P(I),1:Npar+1)
	  End Do
        CALL DSORTQX("A",4,SelectP(1:4,Npar+1),1,Sort1)
	  SelectP=SelectP(Sort1,1:Npar+3)
        P1=SelectP(1,1:Npar+1)
	  P2=SelectP(2,1:Npar+1)
	else
	  P1=Genes0(Parents(G),1:Npar+1)
	  p2=Genes0(Parents(G+1),1:Npar+1)
      endif
  
C Crossover Operations 
      CALL DRNUN (1, choose)
      if (choose .gt. probco) then
	   C1(1:Npar+1)=P1(1:Npar+1)
	   C2(1:Npar+1)=P2(1:Npar+1)
	else
	   if (crossover .eq. 6) then	      
	      Call RNUND(1,5,J)
	   else 
	      J=crossover
	   endif
	   Select Case(J)
	    Case(1) !BLX-alpha=0.5 crossover
	     Do I=1,Npar
	       pmin=Dmin1(P1(I),P2(I))
	       pmax=Dmax1(P1(I),P2(I))
	       delta=pmax-pmin
	       pmin=pmin-0.5*delta
	       pmax=pmax+0.5*delta
	       Call DRNUN(1,lambda)
	       C1(I)=pmin+(pmax-pmin)*lambda
	       Call DRNUN(1,lambda)
	       C2(I)=pmin+(pmax-pmin)*lambda
	     End Do
	    Case(3) ! arithmetic crossover
	       CALL DRNUN(1,lambda)		                
		   C1(1:Npar)=lambda*P1(1:Npar) +
     +                  (1-lambda)*P2(1:Npar)
	       C2(1:Npar)=lambda*P2(1:Npar) +
     +                  (1-lambda)*P1(1:Npar)
	    Case(4) ! Shuffle crossover             
             Do K=1,Npar
                CALL DRNUN (1, lambda)
		      if ( lambda .gt. 0.5) then
	             C1(K)=P2(K)
	             C2(K)=P1(K)
	          else               
                   C1(K)=P1(K)
                   C2(K)=P2(K)
                End IF
	       END DO
	    Case(2) ! Linear crossover
	       Do K=1, Npar
	        C1(K)=0.5*P1(K)+0.5*P2(K)
	        C2(K)=1.5*P1(K)-0.5*P2(K)
	        C3(K)=-.5*P1(K)+1.5*P2(K)
	       End Do
	       Call Crit(Npar,C1(1:Npar), C1(Npar+1))
	       Call Crit(Npar,C2(1:Npar), C2(Npar+1))
             Call Crit(Npar,C3(1:Npar), C3(Npar+1))	       
	       Help(1,1:Npar+1)=C1(1:Npar+1)
		   Help(2,1:Npar+1)=C2(1:Npar+1)
		   Help(3,1:Npar+1)=C3(1:Npar+1)
	       CALL DSORTQX("A",3,Help(1:3,Npar+1),1,Sort2)
	       Help=Help(Sort2,1:Npar+1)	       
	       Do J=1,3
	        if (Help(J,NPar+1) .gt. 0.0) exit
	       END DO
             Select Case(J)
	          Case(1)
			    C1(1:Npar+1)=Help(1,1:Npar+1)
				C2(1:Npar+1)=Help(2,1:Npar+1)
	          Case(2)
	            C1(1:Npar+1)=Help(2,1:Npar+1)
	            C2(1:Npar+1)=Help(3,1:Npar+1)
	          Case(3) 
	            C1(1:Npar+1)=Help(3,1:Npar+1)						
	        End Select
		Case(5)
		   CALL RNUND (1, Npar-1, K)	        
             C1(1:K)=P1(1:K)
	       C1(1+K:Npar)=P2(1+K:Npar)
	       C2(1:K)=P2(1:K)
	       C2(1+K:Npar)=P1(1+K:Npar)                  	   
	   END SELECT
      
	END IF
      
C Mutation operations on C1
      ishere=.false.
      C1(Npar+2)=0.0
      Do J=1,Npar
         CALL DRNUN (1, choose)
	   if (choose .lt. ProbMut(itnum)) then
	      ishere=.true.
	      C1(Npar+2)=C1(Npar+2)+1.0
	      CALL DRNNOR(1, lambda)	   	   	      
	      CALL DRNUN(2,mutvars)
	      add=lambda*
     +	  (1-(mutvars(2)**((1-(DBLE(itnum/Ngen)))**mu3)))
	      if (mutvars(1) .gt. 0.5) then	        
	        C1(J)=C1(J)+add     	
            else  
              C1(J)=C1(J)-add
            End IF
	    END IF
      END DO	
      if (ishere) MutNum(3)=MutNum(3)+1
C Mutation operations on C2
      ishere=.false.
	C2(Npar+2)=0.0
      Do J=1,Npar
         CALL DRNUN (1, choose)
	   if (choose .lt. ProbMut(itnum)) then            
	      ishere=.true.
	      C2(Npar+2)=C2(Npar+2)+1.0
	      CALL DRNNOR(1, lambda)	   	   	      	      
	      CALL DRNUN(2,mutvars)
	      add=lambda*
     +		  (1-(mutvars(2)**((1-(DBLE(itnum/Ngen)))**mu3)))
	      if (mutvars(1) .gt. 0.5) then
	        C2(J)=C2(J)+add     
            else  
              C2(J)=C2(J)-add
            End IF
	    END IF
      END DO
	if (ishere) MutNum(3)=MutNum(3)+1
      MutNum(1)=MutNum(1)+IDINT(C1(Npar+2))+IDINT(C2(Npar+2))

C Determine fitness of children
      Call Crit(Npar,C1(1:Npar), C1(Npar+1))
	Call Crit(Npar,C2(1:Npar), C2(Npar+1))
      
C Selection of family members
      if (selection .eq. 1) then
	   SelectP(1,1:Npar+1)=P1(1:Npar+1)
	   SelectP(1,Npar+2)=0.0 !is parent
	   SelectP(2,1:Npar+1)=P2(1:Npar+1)
	   SelectP(2,Npar+2)=0.0 !is parent
	   SelectP(3,1:Npar+1)=C1(1:Npar+1)
	   SelectP(3,Npar+2)=1.0 !is child
	   SelectP(3,Npar+3)=C1(Npar+2)!with # mutations
	   SelectP(4,1:Npar+1)=C2(1:Npar+1)
	   SelectP(4,Npar+2)=1.0 !is child
	   SelectP(4,Npar+3)=C2(Npar+2)!with # mutations
 
         CALL DSORTQX("A",4,SelectP(1:4,Npar+1),1,Sort1)
	   SelectP=SelectP(Sort1,1:Npar+3)
	   J=1
	   Do while (SelectP(J,Npar+1) .lt. 0.0)
	    J=J+1
	   END DO
      
	   if ((SelectP(J,Npar+2) .gt. 0.0)
     +    .and. (SelectP(J,Npar+3) .gt. 0.0)) then !a mutated child survived
	     MutNum(2)=MutNum(2)+1	      
	   endif

        if ((SelectP(J+1,Npar+2) .gt. 0.0)
     +   .and. (SelectP(J+1,Npar+3) .gt. 0.0)) then !a mutated child survived
	    MutNum(2)=MutNum(2)+1	      
	  endif
C add to new generation  
        Genes1(G,1:Npar+1)=SelectP(J,1:Npar+1)
	  Genes1(G+1,1:Npar+1)=SelectP(J+1,1:Npar+1)

	else

        if (C1(Npar+1) .lt. 0.0) then
  	     Genes1(G,1:Npar+1)=P1(1:Npar+1)
	  else
	    Genes1(G,1:Npar+1)=C1(1:Npar+1)
	  endif
	   
	  if (C2(Npar+1) .lt. 0.0) then
	    Genes1(G+1,1:Npar+1)=P2(1:NPar+1)
	  else
	    Genes1(G+1,1:Npar+1)=C2(1:Npar+1)
	  endif

	endif
      
	END DO

C Elitism
      GMaxMin(1,1)=IDMIN(npop,Genes0(1:npop,Npar+1),1)
	GMaxMin(1,2)=IDMAX(npop,Genes0(1:npop,Npar+1),1)
      GMaxMin(2,1)=IDMIN(npop,Genes1(1:npop,Npar+1),1)
	GMaxMin(2,2)=IDMAX(npop,Genes1(1:npop,Npar+1),1)
		
	CALL SETTEXTPOSITION(12, 15, curpos) 
	Write(fh,102) genes0(GMaxMin(1,1),Npar+1)
	CALL SETTEXTPOSITION(12,45,curpos)
	Write(fh,102)  genes0(GMaxMin(1,2),Npar+1)
	CALL SETTEXTPOSITION(13,15,curpos)
	Write(fh,102)  genes1(GMaxMin(2,1),Npar+1)
	CALL SETTEXTPOSITION(13,45,curpos)
	Write(fh,102)  genes1(GMaxMin(2,2),Npar+1)
      CALL SETTEXTPOSITION(15,1,curpos)
	WRITE(fh,"('Number of mutated elements: ',I4)") MutNum(1)
	Call SetTextPosition(15,35,curpos)	
	WRITE(fh,"('fraction of total: ',F6.3)")
     + (dble(MutNum(1))/dble(npop*npar)) 

	if (selection .eq. 1) then
        CALL SETTEXTPOSITION(16,1,curpos)
	  WRITE(fh,"('Number of survived mutants: ',I4)") MutNum(2)
   	  CALL SETTEXTPOSITION(16,35,curpos)
 	  WRITE(fh,"('fraction of total: ',F6.3)")
     + (dble(MutNum(2))/dble(MutNum(3))) 
	endif 

C elitism
	 
	if (Genes1(GMaxMin(2,1),Npar+1) .gt.
     +	 Genes0(GMaxMin(1,1),Npar+1)) then
	   Genes1(GMaxMin(2,2),1:Npar+1)=
     +	   Genes0(GMaxMin(1,1),1:Npar+1)	   
	endif
	Genes0=Genes1      
      	
	END DO

      Par=Genes1(GMaxMin(2,1),1:Npar)
      Fit=Genes1(GMaxMin(2,1),Npar+1)
      
102   Format(E25.6E3)
      

      Return
      END SUBROUTINE



